! Calculation of phonon

subroutine calc_phonon
  use variables
  use constants
  use variables_inst
  double precision,allocatable :: rx_s(:),ry_s(:),rz_s(:)
  double precision :: hmat_s(3,3)
  double precision :: inf_str, inf_dis
  double precision,allocatable :: dfx(:),dfy(:),dfz(:),tmp(:)
  double precision,allocatable :: hessian(:,:)
  double precision :: emin,emax,xx
  double precision,allocatable :: vec(:)
  character(13) :: filename
  
  double precision :: rlb=-1.0d100, rub=1.0d100
  double precision,allocatable :: AR(:,:),AI(:,:),EVR(:,:),EVI(:,:) &
       ,E(:),EI(:),FV1(:),FV2(:),FV3(:),FV4(:),FV5(:),FV6(:),FV7(:),FV8(:),FM1(:,:),IV1(:) &
       ,WORK(:)
  integer, allocatable :: nod(:),nindx(:)
  integer :: irmv(6)
  integer :: ipx, ipy, ipz
  
!!! degree of freedom
!  nofpbc=0
!  if (pbcx) nofpbc=nofpbc+1; if (pbcy) nofpbc=nofpbc+1; if (pbcz) nofpbc=nofpbc+1
!  if (nofpbc==1) then
!     if (iedg==icnt) nn=n*3-3
!     if (iedg/=icnt) nn=n*3-4
!  endif
!  if (nofpbc==2) nn=n*3-3;   if (nofpbc==3) nn=n*3-3
!  if (nofpbc==0) stop 'instability.F: NO PBC is currently not supported.'
  
  npx = nphonon_x; npy = nphonon_y; npz = nphonon_z
  n0 = n; n = n*npx*npy*npz
  nn = n0*3

  allocate (ar(nn,nn),ai(nn,nn),evr(nn,nn),evi(nn,nn),e(nn),ei(nn))
  allocate (fv1(nn),fv2(nn),fv3(nn),fv4(nn),fv5(nn))
  allocate (fv6(nn),fv7(nn),fv8(nn),fm1(2,nn),iv1(nn))
  allocate (work(nn*4))
  
  allocate (rx_s(n0),ry_s(n0),rz_s(n0))
  allocate (hessian(nn,nn))
  allocate (vec(nn))
  allocate (nindx(nn))
  
  hmat_s=hmat ! Cell size save
  do i=1,n0
     rx_s(i)=rx(i); ry_s(i)=ry(i); rz_s(i)=rz(i) ! Atom position save
  enddo
  
!!! Multiplication of simulation cell.
!!! Note: 1st cell vector is (h11,h21,h31)
!!! Note: 2nd cell vector is (h12,h22,h32)
!!! Note: 3rd cell vector is (h13,h23,h33)
!!!                h11 h12 h13      |   |   |
!!! Note: hmat is  h21 h22 h23  =  1st 2nd 3rd
!!!                h31 h32 h33      |   |   |
  deallocate (rx,ry,rz,fx,fy,fz,asp)
  allocate (rx(n),ry(n),rz(n),fx(n),fy(n),fz(n),asp(n))
  if (npx>1) then
     do iix=2,npx
        do i=1,n0
           ii=(iix-1)*n0+i
           rx(ii)=rx(i)+hmat(1,1)*dble(iix-1)
           ry(ii)=ry(i)+hmat(2,1)*dble(iix-1)
           rz(ii)=rz(i)+hmat(3,1)*dble(iix-1)
           asp(ii)=asp(i);mfx(ii)=mfx(i);mfy(ii)=mfy(i);mfz(ii)=mfz(i)
        enddo
     enddo
     n0=n0*npx
     hmat(1,1)=hmat(1,1)*dble(npx)
     hmat(2,1)=hmat(2,1)*dble(npx)
     hmat(3,1)=hmat(3,1)*dble(npx)
  endif
  if (npy>1) then
     do iiy=2,npy
        do i=1,n0
           ii=(iiy-1)*n0+i
           rx(ii)=rx(i)+hmat(1,2)*dble(iiy-1)
           ry(ii)=ry(i)+hmat(2,2)*dble(iiy-1)
           rz(ii)=rz(i)+hmat(3,2)*dble(iiy-1)
           asp(ii)=asp(i);mfx(ii)=mfx(i);mfy(ii)=mfy(i);mfz(ii)=mfz(i)
        enddo
     enddo
     n0=n0*npy
     hmat(1,2)=hmat(1,2)*dble(npy)
     hmat(2,2)=hmat(2,2)*dble(npy)
     hmat(3,2)=hmat(3,2)*dble(npy)
  endif
  if (npz>1) then
     do iiz=2,npz
        do i=1,n0
           ii=(iiz-1)*n0+i
           rx(ii)=rx(i)+hmat(1,3)*dble(iiz-1)
           ry(ii)=ry(i)+hmat(2,3)*dble(iiz-1)
           rz(ii)=rz(i)+hmat(3,3)*dble(iiz-1)
           asp(ii)=asp(i);mfx(ii)=mfx(i);mfy(ii)=mfy(i);mfz(ii)=mfz(i)
        enddo
     enddo
     n0=n0*npx
     hmat(1,3)=hmat(1,3)*dble(npz)
     hmat(2,3)=hmat(2,3)*dble(npz)
     hmat(3,3)=hmat(3,3)*dble(npz)
  endif
!!! End of cell multiplication.

  inf_dis=0.05*ang        ! Infinitesimal displacement
  !      inf_dis=0.0002*ang        ! Infinitesimal displacement

!!!  Hessian: atom coordinates term
  print *,'Phonon: Making force constant (Hessian) matrix..'
  do i=1,n
     rx(i)=rx(i)+inf_dis
     call potential
     do j=1,n
        hessian((i-1)*3+1,(j-1)*3+1)=fx(j)
        hessian((i-1)*3+1,(j-1)*3+2)=fy(j)
        hessian((i-1)*3+1,(j-1)*3+3)=fz(j)
     enddo
     rx(i)=rx(i)-inf_dis*2.0
     call potential
     do j=1,n
        hessian((i-1)*3+1,(j-1)*3+1)=fx(j)-hessian((i-1)*3+1,(j-1)*3+1)
        hessian((i-1)*3+1,(j-1)*3+2)=fy(j)-hessian((i-1)*3+1,(j-1)*3+2)
        hessian((i-1)*3+1,(j-1)*3+3)=fz(j)-hessian((i-1)*3+1,(j-1)*3+3)
     enddo
     rx(i)=rx(i)+inf_dis
     
     ry(i)=ry(i)+inf_dis
     call potential
     do j=1,n
        hessian((i-1)*3+2,(j-1)*3+1)=fx(j)
        hessian((i-1)*3+2,(j-1)*3+2)=fy(j)
        hessian((i-1)*3+2,(j-1)*3+3)=fz(j)
     enddo
     ry(i)=ry(i)-inf_dis*2.0
     call potential
     do j=1,n
        hessian((i-1)*3+2,(j-1)*3+1)=fx(j)-hessian((i-1)*3+2,(j-1)*3+1)
        hessian((i-1)*3+2,(j-1)*3+2)=fy(j)-hessian((i-1)*3+2,(j-1)*3+2)
        hessian((i-1)*3+2,(j-1)*3+3)=fz(j)-hessian((i-1)*3+2,(j-1)*3+3)
     enddo
     ry(i)=ry(i)+inf_dis         
     
     rz(i)=rz(i)+inf_dis
     call potential
     do j=1,n
        hessian((i-1)*3+3,(j-1)*3+1)=fx(j)
        hessian((i-1)*3+3,(j-1)*3+2)=fy(j)
        hessian((i-1)*3+3,(j-1)*3+3)=fz(j)
     enddo
     rz(i)=rz(i)-inf_dis*2.0
     call potential
     do j=1,n
        hessian((i-1)*3+3,(j-1)*3+1)=fx(j)-hessian((i-1)*3+3,(j-1)*3+1)
        hessian((i-1)*3+3,(j-1)*3+2)=fy(j)-hessian((i-1)*3+3,(j-1)*3+2)
        hessian((i-1)*3+3,(j-1)*3+3)=fz(j)-hessian((i-1)*3+3,(j-1)*3+3)
     enddo
     rz(i)=rz(i)+inf_dis         
  enddo
  
  hessian=hessian/inf_dis/2.0

      emax=-1.0d100
      do i=1,nn; do j=1,nn
         emax=max(hessian(i,j),emax)
      enddo; enddo
      hessian=hessian/emax

      if (.false.) then
         print *,'### HESIAN MATRIX ###'
         do i=1,nn; do j=1,nn
            print *,i,j,hessian(i,j)
         enddo; enddo
      endif

! search one atom at edge and one near cell center
      if (iedg==0) then
         xmin=1.0d10; ymin=1.0d10; zmin=1.0d10
         do i=1,n
            if (rx(i)<xmin) then
               xmin=rx(i); iedg=i
            endif
            if (ry(i)<ymin) then
               ymin=ry(i); iedg=i
            endif
            if (rz(i)<zmin) then
               zmin=rz(i); iedg=i
            endif
         enddo
         xcnt=(hmat(1,1)+hmat(2,1)+hmat(3,1))/2.0
         ycnt=(hmat(1,2)+hmat(2,2)+hmat(3,2))/2.0
         zcnt=(hmat(1,3)+hmat(2,3)+hmat(3,3))/2.0
         rng=1.0d-11
888      icnt=0
         do i=1,n
            if ((abs(rx(i)-xcnt)<rng).and.(abs(ry(i)-ycnt)<rng).and.(abs(rz(i)-zcnt)<rng)) then
               icnt=i; exit
            endif
         enddo
         if (icnt==0) then
            rng=rng*2.0; go to 888
         endif
      endif
      
      print *,'Edge atom=',iedg,'(',real(rx(iedg)*1.0e10),real(ry(iedg)*1.0e10),real(rz(iedg)*1.0e10),') [ang]'
      print *,'Cent atom=',icnt,'(',real(rx(icnt)*1.0e10),real(ry(icnt)*1.0e10),real(rz(icnt)*1.0e10),') [ang]'

!      iedg=1; icnt=7
!!! --- Matrix set (removing some lines to meet DoF) ---
      if (nofpbc==3) then      ! --> atom icnt is fixed
         nrmv=3; irmv(1)=3*(icnt-1)+1; irmv(2)=3*(icnt-1)+2; irmv(3)=3*(icnt-1)+3
      else if (nofpbc==2) then ! --> atom icnt is fixed 
         nrmv=3; irmv(1)=3*(icnt-1)+1; irmv(2)=3*(icnt-1)+2; irmv(3)=3*(icnt-1)+3
      else if (nofpbc==1) then ! --> atom icnt is fixed, iedg is constrained
         if (iedg/=icnt) then
            nrmv=4; irmv(1)=3*(icnt-1)+1; irmv(2)=3*(icnt-1)+2; irmv(3)=3*(icnt-1)+3
            if (pbcx) irmv(4)=3*(iedg-1)
            if (pbcy) irmv(4)=3*(iedg-1)+1
            if (pbcz) irmv(4)=3*(iedg-1)+2
         else
            nrmv=3; irmv(1)=3*(icnt-1)+1; irmv(2)=3*(icnt-1)+2; irmv(3)=3*(icnt-1)+3
         endif
      endif

      indx=0
      do i=1,n*3
         do ii=1,nrmv
            if (i==irmv(ii)) go to 990
         enddo
         indx=indx+1
         nindx(indx)=i

         jndx=0
         do j=1,n*3
            do ii=1,nrmv
               if (j==irmv(ii)) go to 991
            enddo
            jndx=jndx+1

            ar(indx,jndx)=hessian(i,j)
            
991         continue
         enddo

990      continue
      enddo
      ai=0

!      do i=1,nn
!         write (6,'(6f12.5)') (ar(i,j),j=1,6)
!      enddo

      noc=nn
      print *,'Solving eigenvalue problem for atomonly matrix..'
!      CALL EISPC(nn,nn,AR,AI,E,EVR,EVI,RLB,RUB,FV1,FV2,FV3 &
!      ,FV4,FV5,FV6,FV7,FV8,FM1,IV1,NOC)
      call DGEEV('N','V',nn,ar,nn,e,ei,evi,nn,evr,nn,work,nn*4,INFO)
!      call DGEEV('V','N',nn,ar,nn,e,ei,evr,nn,evi,nn,work,nn*4,INFO)

! Sort eigenvalues
!      write (6,'(6f10.5)') (e(j),j=1,6)

      allocate (nod(nn))
      do i=1,nn
         nod(i)=i
      end do
      call sort(nn,e,nod)
!      print *,nod(1),nod(2),nod(3),nod(4),nod(5)

!      write (6,'(6f10.3)') (e(j),j=1,6)
!      write (6,*)
!      write (6,'(6f10.3)') (evr(j,nod(1)),j=1,6)
!      write (6,'(6f10.3)') (evr(j,nod(2)),j=1,6)
!      write (6,'(6f10.3)') (evr(j,nod(3)),j=1,6)
!      write (6,'(6f10.3)') (evr(j,nod(4)),j=1,6)
!      write (6,'(6f10.3)') (evr(j,nod(5)),j=1,6)
!      write (6,'(6f10.3)') (evr(j,nod(6)),j=1,6)

      write (6,'("### INSTABILITY ANALYSIS (ATOMONLY) ###")')
      if (e(1)<0.0) write (6,'("Negative eigenvalue found:",e12.4)') e(1)
      write (6,'("MINIMUM EIGENVALUE (ATOM):",e12.4)') e(1)

      open (20,FILE='emin.d')
      open (21,FILE='eminall.d')
      if (continue_md) then
         do while(.true.)
            read (20,*,end=111) xx
         enddo
111      continue
      endif
      write (20,'(i8,10f10.5)') step,e(1),e(2),e(3),e(4),e(5),e(6),e(7),e(8),e(9),e(10)
      do i=1,nn
         write (21,*) i,e(i)
      enddo
      close (20)
      close (21)

! output data
      do i=1,20
!         xx=0.0d0
!         do k=1,n-1
!            fx(k)=evr((k-1)*3+1,i)
!            fy(k)=evr((k-1)*3+2,i)
!            fz(k)=evr((k-1)*3+3,i)
!            xx=max(abs(fx(k)),xx)
!            xx=max(abs(fy(k)),xx)
!            xx=max(abs(fz(k)),xx)
!         enddo
!         fx(n)=0.0d0; fy(n)=0.0d0; fz(n)=0.0d0
!         vec=sqrt(fx**2+fy**2+fz**2)

         fx=0.0; fy=0.0; fz=0.0
         xx=0.0d0
         do j=1,nn
            jj=nindx(j)
            if (jj/=0) then
               if (mod(jj,3)==1) then
                  jjj=(jj-1)/3+1; fx(jjj)=evr(j,nod(i))
               endif
               if (mod(jj,3)==2) then
                  jjj=(jj-2)/3+1; fy(jjj)=evr(j,nod(i))
               endif
               if (mod(jj,3)==0) then
                  jjj=(jj-3)/3+1; fz(jjj)=evr(j,nod(i))
               endif
               xx=max(abs(evr(j,nod(i))),xx)
            endif
         end do
         vec=sqrt(fx**2+fy**2+fz**2)

! write **.xsf
         xx=xx*5.0d8
         fx=fx/xx; fy=fy/xx; fz=fz/xx
         if (i== 1) call write_xsf(30,'atominst01.xsf')
         if (i== 2) call write_xsf(30,'atominst02.xsf')
         if (i== 3) call write_xsf(30,'atominst03.xsf')
         if (i== 4) call write_xsf(30,'atominst04.xsf')
         if (i== 5) call write_xsf(30,'atominst05.xsf')
         if (i== 6) call write_xsf(30,'atominst06.xsf')
         if (i== 7) call write_xsf(30,'atominst07.xsf')
         if (i== 8) call write_xsf(30,'atominst08.xsf')
         if (i== 9) call write_xsf(30,'atominst09.xsf')
         if (i==10) call write_xsf(30,'atominst10.xsf')
         if (i==11) call write_xsf(30,'atominst11.xsf')
         if (i==12) call write_xsf(30,'atominst12.xsf')
         if (i==13) call write_xsf(30,'atominst13.xsf')
         if (i==14) call write_xsf(30,'atominst14.xsf')
         if (i==15) call write_xsf(30,'atominst15.xsf')
         if (i==16) call write_xsf(30,'atominst16.xsf')
         if (i==17) call write_xsf(30,'atominst17.xsf')
         if (i==18) call write_xsf(30,'atominst18.xsf')
         if (i==19) call write_xsf(30,'atominst19.xsf')
         if (i==20) call write_xsf(30,'atominst20.xsf')

      end do
    end subroutine calc_phonon

