!
!   EAM FORCE & ENERGY CALCULATION
!   for Aluminum/Copper/Nickel
!   Mishin potential
!   calculation of force, total energy, (d(mat) <- for extension)
!   New bookkeep algorithm
!   
!   Feb. 24 (2006)
!   Stomic stress calculation is implemented
!   Oct. 5, 2010
!   A bug in stress calculation for a small cell is fixed
!   July 18, 2012
!   The stepwise mesh data are smoothened by linear interpolation (trapezoid method),
!    which should work for more precise MD or relaxation (e.g. Fmax<1.0e-6)

      module mishin_function
      double precision, allocatable :: pairx(:),pairy(:),pairc(:,:),pairxn(:)
      double precision, allocatable :: denx(:),deny(:),denc(:,:),denxn(:)
      double precision, allocatable :: embedx(:),embedy(:),embedc(:,:),embedxn(:)
      double precision, allocatable :: spd2(:)
      integer, parameter :: mesh=1000000
      double precision :: pair(mesh),paird(mesh)
      double precision :: den(mesh),dend(mesh)
      double precision :: embed(mesh),embedd(mesh)
      dimension DF(2), IOPT(2), EC(4)
      double precision :: pairmin,pairmax
      double precision :: denmin,denmax
      double precision :: embedmin,embedmax
      logical, save :: mishin_setup_done = .false.
      end module

      subroutine eam_Mishin
      use mishin_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: rhob(n) ! rho-bar_i (density)
      double precision :: ki,kif
      integer :: istat

      istat=1

!  Functions setup
      if (.not.mishin_setup_done) then
         print *,'mishin setup'
         call mishin_setup
      endif

      if (rc<0.0) then
         stop 'Fatal error: please specify rc in SETDAT'
      endif
      rc2=rc**2

      rhob=0.0d0

!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!   First Loop: Chg dns calc
      kns=0
      idbg=0
      do 4200 k=1,n
         kn=knum(k)
         if (kn.eq.0) go to 4200
         do 4300 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(k)-rx(j)
            yy=ry(k)-ry(j)
            zz=rz(k)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
!                  ii=int(dble(mesh)*(r/ang-denmin)/(denmax-denmin))
                  xi=dble(mesh)*(r/ang-denmin)/(denmax-denmin)
                  ii=int(xi)
!                  rhob(k)=rhob(k)+den(ii)
                  rhob(k)=rhob(k)+den(ii)+(den(ii+1)-den(ii))*(xi-dble(ii))
                  if (j/=k) then
!                     rhob(j)=rhob(j)+den(ii)
                     rhob(j)=rhob(j)+den(ii)+(den(ii+1)-den(ii))*(xi-dble(ii))
                  endif
               endif
            endif
 4300    continue
 4200 continue

!     Calculation of potential energy
      do i=1,n
!         ii=int(dble(mesh)*(rhob(i)-embedmin)/(embedmax-embedmin))
         xi=dble(mesh)*(rhob(i)-embedmin)/(embedmax-embedmin)
         ii=int(xi)
!         enpoteach(i)=embed(ii)*ev
         enpoteach(i)=(embed(ii)+(embed(ii+1)-embed(ii))*(xi-dble(ii)))*ev
      enddo

!   Second Loop: Force calc
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      do 4500 i=1,n
         kn=knum(i)
         if (kn.eq.0) go to 4500
         do 4600 jk=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j)
            yy=ry(i)-ry(j)
            zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            if (istat == 1) then
               r2=xx**2+yy**2+zz**2
               if (r2.lt.rc2) then
                  r=sqrt(r2)
!                  ii=int(dble(mesh)*(r/ang-pairmin)/(pairmax-pairmin))
                  xi=dble(mesh)*(r/ang-pairmin)/(pairmax-pairmin)
                  ii=int(xi)
!                  vij=pair(ii)*ev
                  vij=(pair(ii)+(pair(ii+1)-pair(ii))*(xi-dble(ii)))*ev
!                  dv=paird(ii)
                  dv=paird(ii)+(paird(ii+1)-paird(ii))*(xi-dble(ii))
                  enpoteach(i)=enpoteach(i)+vij/2.0d0
                  vijsum=vijsum+vij/2.0d0
                  if (j/=i) then
                     enpoteach(j)=enpoteach(j)+vij/2.0d0
                     vijsum=vijsum+vij/2.0d0
                  endif

!                  ii=int(dble(mesh)*(rhob(i)-embedmin)/(embedmax-embedmin))
!                  jj=int(dble(mesh)*(rhob(j)-embedmin)/(embedmax-embedmin))
                  xi=dble(mesh)*(rhob(i)-embedmin)/(embedmax-embedmin)
                  xj=dble(mesh)*(rhob(j)-embedmin)/(embedmax-embedmin)
                  ii=int(xi); jj=int(xj)
!                  df_i=embedd(ii)
!                  df_j=embedd(jj)
                  df_i=embedd(ii)+(embedd(ii+1)-embedd(ii))*(xi-dble(ii))
                  df_j=embedd(jj)+(embedd(jj+1)-embedd(jj))*(xj-dble(jj))

!                  ii=int(dble(mesh)*(r/ang-denmin)/(denmax-denmin))
                  xi=dble(mesh)*(r/ang-denmin)/(denmax-denmin)
                  ii=int(xi)
!                  dr=dend(ii)
                  dr=dend(ii)+(dend(ii+1)-dend(ii))*(xi-dble(ii))

                  f=-(df_i*dr+df_j*dr+dv)*ev/ang

                  ki=f/r
                  if (j==i) ki=ki/2.0d0
                  kif=ki/2.0d0
                  fx(i)=fx(i)+ki*xx
                  fy(i)=fy(i)+ki*yy
                  fz(i)=fz(i)+ki*zz
                  fx(j)=fx(j)-ki*xx
                  fy(j)=fy(j)-ki*yy
                  fz(j)=fz(j)-ki*zz
                  satom(i,1,1)=satom(i,1,1)-kif*xx*xx
                  satom(i,1,2)=satom(i,1,2)-kif*yy*xx
                  satom(i,2,2)=satom(i,2,2)-kif*yy*yy
                  satom(i,1,3)=satom(i,1,3)-kif*zz*xx
                  satom(i,2,3)=satom(i,2,3)-kif*zz*yy
                  satom(i,3,3)=satom(i,3,3)-kif*zz*zz
                  satom(j,1,1)=satom(j,1,1)-kif*xx*xx
                  satom(j,1,2)=satom(j,1,2)-kif*yy*xx
                  satom(j,2,2)=satom(j,2,2)-kif*yy*yy
                  satom(j,1,3)=satom(j,1,3)-kif*zz*xx
                  satom(j,2,3)=satom(j,2,3)-kif*zz*yy
                  satom(j,3,3)=satom(j,3,3)-kif*zz*zz
               endif
            endif
 4600    continue
 4500 continue

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

!      write (10,*) 'EMT force calculation end.'
      end



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
!     MISHIN SETUP
!     READ FUNCTIONS
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!      
      subroutine mishin_setup
      use mishin_function
      use constants
      implicit double precision(a-h,o-z)
      character (1) :: ichar
      
! Pair function
      if ((asp(1)=='cu').or.(asp(1)=='Cu')) then
         open (10,file='pot/EAMPARAM.PRB63-224106.cu.pair')
      else if ((asp(1)=='ni').or.(asp(1)=='Ni')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.ni.pair')
      else if ((asp(1)=='al').or.(asp(1)=='Al')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.al.pair')
      else 
         print *,'eam_Mishin is prepared only for Cu, Ni and Al'
         print *, asp(1),' is not supported'
         stop
      endif
      mesh_pair=0
 10   read (10,*,end=30) ichar !trial read
      if (ichar(:1)=='#') then
         go to 10               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
 20   read (10,*,end=30,err=10) a1,a2
      mesh_pair=mesh_pair+1
      go to 20
 30  close (10)
      allocate(pairx(mesh_pair),pairy(mesh_pair),pairc(mesh_pair,3))
      if ((asp(1)=='cu').or.(asp(1)=='Cu')) then
         open (10,file='pot/EAMPARAM.PRB63-224106.cu.pair')
      else if ((asp(1)=='ni').or.(asp(1)=='Ni')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.ni.pair')
      else if ((asp(1)=='al').or.(asp(1)=='Al')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.al.pair')
      else 
         print *,'Only Cu, Ni and Al are supported.'
         stop
      endif
      mesh_pair=0
 15   read (10,*,end=35) ichar !trial read
      if (ichar(:1)=='#') then
         go to 15               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
 25   read (10,*,end=35,err=15) a1,a2
      mesh_pair=mesh_pair+1
      if (mesh_pair==1) pairmin=a1
      pairx(mesh_pair)=a1
      pairy(mesh_pair)=a2
      go to 25
 35   close (10)
      pairmax=a1
! finer mesh set
      allocate(pairxn(mesh),spd2(mesh))
      x=pairmin
      d=(pairmax-pairmin)/dble(mesh-1)
      do i=1,mesh
         pairxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3
      iopt(2)=3
      call SPLC(pairx, mesh_pair, pairy, DF, IOPT, pairc, mesh_pair, IER)
      call SPLF(pairx,mesh_pair,pairy,pairc,mesh_pair,pairxn,mesh,pair, IER)
      call SPLD(pairx,mesh_pair,pairc,mesh_pair,pairxn,mesh,paird,spd2, IER)
      deallocate (pairxn,spd2)
      deallocate(pairx,pairy,pairc)

! Cutoff radius set
      rc=pairmax*ang
      rc2=rc**2
      print *,'EAM Mishin: rc=',real(rc*1.0e10),'  ang'

! density function
      if ((asp(1)=='cu').or.(asp(1)=='Cu')) then
         open (10,file='pot/EAMPARAM.PRB63-224106.cu.den')
      else if ((asp(1)=='ni').or.(asp(1)=='Ni')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.ni.den')
      else if ((asp(1)=='al').or.(asp(1)=='Al')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.al.den')
      else 
         print *,'Only Cu, Ni and Al are supported.'
         stop
      endif
      mesh_den=0
 110  read (10,*,end=130) ichar !trial read
      if (ichar(:1)=='#') then
         go to 110               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
 120  read (10,*,end=130,err=110) a1,a2
      mesh_den=mesh_den+1
      go to 120
 130  close (10)
      allocate(denx(mesh_den),deny(mesh_den),denc(mesh_den,3))
      if ((asp(1)=='cu').or.(asp(1)=='Cu')) then
         open (10,file='pot/EAMPARAM.PRB63-224106.cu.den')
      else if ((asp(1)=='ni').or.(asp(1)=='Ni')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.ni.den')
      else if ((asp(1)=='al').or.(asp(1)=='Al')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.al.den')
      else 
         print *,'Only Cu, Ni and Al are supported.'
         stop
      endif
      mesh_den=0
 115  read (10,*,end=135) ichar !trial read
      if (ichar(:1)=='#') then
         go to 115               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
 125  read (10,*,end=135,err=115) a1,a2
      mesh_den=mesh_den+1
      if (mesh_den==1) denmin=a1
      denx(mesh_den)=a1
      deny(mesh_den)=a2
      go to 125
 135  close (10)
      denmax=a1
! finer mesh set
      allocate(denxn(mesh),spd2(mesh))
      x=denmin
      d=(denmax-denmin)/dble(mesh-1)
      do i=1,mesh
         denxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3
      iopt(2)=3
      call SPLC(denx, mesh_den, deny, DF, IOPT, denc, mesh_den, IER)
      call SPLF(denx,mesh_den,deny,denc,mesh_den,denxn,mesh,den, IER)
      call SPLD(denx,mesh_den,denc,mesh_den,denxn,mesh,dend,spd2, IER)
      deallocate (denxn,spd2)
      deallocate(denx,deny,denc)

! Embed function
      if ((asp(1)=='cu').or.(asp(1)=='Cu')) then
         open (10,file='pot/EAMPARAM.PRB63-224106.cu.embed')
      else if ((asp(1)=='ni').or.(asp(1)=='Ni')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.ni.embed')
      else if ((asp(1)=='al').or.(asp(1)=='Al')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.al.embed')
      else 
         print *,'Only Cu, Ni and Al are supported.'
         stop
      endif
      mesh_embed=0
 210  read (10,*,end=230) ichar !trial read
      if (ichar(:1)=='#') then
         go to 210               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
 220  read (10,*,end=230,err=210) a1,a2
      mesh_embed=mesh_embed+1
      go to 220
 230  close (10)
      allocate(embedx(mesh_embed),embedy(mesh_embed),embedc(mesh_embed,3))
      if ((asp(1)=='cu').or.(asp(1)=='Cu')) then
         open (10,file='pot/EAMPARAM.PRB63-224106.cu.embed')
      else if ((asp(1)=='ni').or.(asp(1)=='Ni')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.ni.embed')
      else if ((asp(1)=='al').or.(asp(1)=='Al')) then
         open (10,file='pot/EAMPARAM.PRB59-3393.al.embed')
      else 
         print *,'Only Cu, Ni and Al are supported.'
         stop
      endif
      mesh_embed=0
 215  read (10,*,end=235) ichar !trial read
      if (ichar(:1)=='#') then
         go to 215               !skip a line if it is a comment line
      else
         backspace 10           !read again if it is not a comment line
      endif
 225  read (10,*,end=235,err=215) a1,a2
      mesh_embed=mesh_embed+1
      if (mesh_embed==1) embedmin=a1
      embedx(mesh_embed)=a1
      embedy(mesh_embed)=a2
      go to 225
 235  close (10)
      embedmax=a1
! finer mesh set
      allocate(embedxn(mesh),spd2(mesh))
      x=embedmin
      d=(embedmax-embedmin)/dble(mesh-1)
      do i=1,mesh
         embedxn(i)=x
         x=x+d
      enddo
! function set
      iopt(1)=3
      iopt(2)=3
      call SPLC(embedx, mesh_embed, embedy, DF, IOPT, embedc, mesh_embed, IER)
      call SPLF(embedx,mesh_embed,embedy,embedc,mesh_embed,embedxn,mesh,embed, IER)
      call SPLD(embedx,mesh_embed,embedc,mesh_embed,embedxn,mesh,embedd,spd2, IER)
      deallocate (embedxn,spd2)
      deallocate(embedx,embedy,embedc)

      mishin_setup_done=.true.

      end

      
