!
! Potential optimizer using genetic algorithm
! (Modified Jan. 08 -)
!
    module fit_variables
      integer, parameter :: nbit = 24 ! number of bits
      integer, parameter :: np = 30   ! population size
      integer, parameter :: nval = 2  ! number of variables
      integer, allocatable :: chrom(:,:),best(:),gbest(:)
      integer :: ngbest
      double precision :: crate,mrate,valmax0,valmin0
      integer, allocatable :: n21(:),ns(:)
      double precision, allocatable :: val(:),q(:)
      double precision, allocatable :: xrmin(:),xrmax(:)
      integer :: mode(2)
      integer, allocatable :: newch(:,:),n21g(:)
      integer, parameter :: ngen = 100
!      integer :: ng
    end module fit_variables

    program pfga
      use fit_variables
      use variables
      use constants
      use output
      use bookkeep_param
      use rp_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: x0, x1
      character*1 fn1(2),fn2(2)
      character*11 fname(6)
      double precision :: xx(2)
      common ng
      
      allocate (chrom(np,nbit*nval),best(nbit*nval),gbest(nbit*nval))
      allocate (n21(nbit),val(np),q(np),ns(np))
      allocate (xrmin(nval),xrmax(nval))
      allocate (newch(np,nbit*nval),n21g(nbit))

      data fn1 / 'b', 'g' /
      data fn2 / 's', 'm' /
      
      ! Range of variables (parameter-space to be searched)
      xrmin(1)=0.10d0; xrmax(1)=0.40d0
      xrmin(2)=0.70d0; xrmax(2)=1.30d0

!      crate : crossover rate
!      mrate : mutation rate
      crate=0.25d0
      mrate=0.01d0
!      mode(1)=0 : binary code , =1 : gray code
!      mode(2)=0 : single point crossover
!      mode(2)=1 : multi point crossover
      
      mode(1)=1
      mode(2)=0

!      x0 : random number seed
      x0=15896.0d0

!      definition and open of output file
      fname(1)='data2.dat'
      open(1,file=fname(1))
      fname(2)='lmin2.xy'
      open(2,file=fname(2))
      fname(3)='aver2.xy'
      open(3,file=fname(3))
      fname(4)='gmin2.xy'
      open(4,file=fname(4))
      fname(5)='x2_500.xy'
      open(7,file=fname(5))
      fname(6)='x2_50.xy'
      open(8,file=fname(6))
      
      do i=2,4
        do j=1,4
        write(i,*)
        enddo
      enddo
      do i=7,8
        do j=1,4
        write(i,*)
        enddo
      enddo
      
      call init1(x0)
      
      valmax0=-2.0d0
      valmin0= 2.0d0
      


!=============================================================================
!----------------------------------------------------
!----- SETTING --------------------------------------
!----------------------------------------------------
!---INPUT VALUE---
!---PBC
! RECOMMENDED: (T T F) for 2-D, (T F F) for 1-D
      pbcx = .true.
      pbcy = .true.
      pbcz = .true.

!---STRESS & TEMPERATURE CONTROL
      sgm_set      =  0.0d6     ! Stress (Pa) (Positive is tension)
      temp_set     =  100.0d0      ! Temperature (K)
      step_ite     =  100
      algorithm    =  0
      vscale       =  0         ! Velocity scaling
      damper       =  0         ! Atom damping
      cellconstx   =  .false.   ! Cell constraint
      cellconsty   =  .false.   
      cellconstz   =  .false.   
      
!  Stress set
      sxx=0.0d6; syy=0.0d6; szz=0.0d6 ! <-- Edit here
      syz=0.0d6; szx=0.0d6; sxy=0.0d6 ! <-- Edit here
      sgmmat_set   = 0.0d0
      sgmmat_set(1,1) = sxx
      sgmmat_set(2,2) = syy
      sgmmat_set(3,3) = szz
      sgmmat_set(2,3) = syz !/2.0d0
      sgmmat_set(1,3) = szx !/2.0d0
      sgmmat_set(1,2) = sxy !/2.0d0
      sgmmat_set(3,2) = sgmmat_set(2,3)
      sgmmat_set(3,1) = sgmmat_set(1,3)
      sgmmat_set(2,1) = sgmmat_set(1,2)
      if (.not.pbcx) sgmmat_set(1,1)=0.0d0
      if (.not.pbcy) sgmmat_set(2,2)=0.0d0
      if (.not.pbcz) sgmmat_set(3,3)=0.0d0
      
!  Strain set
      exx=0.0d0; eyy=0.0d0; ezz=0.0d0 ! <-- Edit here
      eyz=0.0d0; ezx=0.0d0; exy=0.0d0 ! <-- Edit here
      strmat_set = 0.0d0
      strmat_set(1,1) = exx
      strmat_set(2,2) = eyy
      strmat_set(3,3) = ezz
      strmat_set(2,3) = eyz/2.0d0
      strmat_set(1,3) = ezx/2.0d0
      strmat_set(1,2) = exy/2.0d0
      strmat_set(3,2) = strmat_set(2,3)
      strmat_set(3,1) = strmat_set(1,3)
      strmat_set(2,1) = strmat_set(1,2)

!---SYSTEM SETTING
!      a=4.04959d-10
      a=3.61d-10
!      wm=am*13.0d0+bm*14.0d0
      call initconfsc(5,5,5)    ! Single crystal
      call strain               ! Apply strain matrix

!----- SETTING END ---------------------------------

!------------------------------------------------------
! Initialize variables
      call var_initialize

      call readcondition
!      if (continue_md) then
!         call readcont
!      else
         call readconfig
!         if (ini_velocity) call initial_velocity
!      endif
      hmat0 = hmat

! Set atom species
      call set_atom_species
! Weight of atom
      call atom_weight
      if (.not.continue_md) then
         if (ini_velocity) call initial_velocity
      endif
!-----
      step_start=step
      step_end=step+step_ite
      rc2=rc**2
      dt2=dt**2

!  Stress set
      sgmmat_set   = 0.0d0
      sgmmat_set(1,1) = sxx
      sgmmat_set(2,2) = syy
      sgmmat_set(3,3) = szz
      sgmmat_set(2,3) = syz !/2.0d0
      sgmmat_set(1,3) = szx !/2.0d0
      sgmmat_set(1,2) = sxy !/2.0d0
      sgmmat_set(3,2) = sgmmat_set(2,3)
      sgmmat_set(3,1) = sgmmat_set(1,3)
      sgmmat_set(2,1) = sgmmat_set(1,2)
      if (.not.pbcx) sgmmat_set(1,1)=0.0d0
      if (.not.pbcy) sgmmat_set(2,2)=0.0d0
      if (.not.pbcz) sgmmat_set(3,3)=0.0d0

      pex=-sgm_set
      pexmat=-sgmmat_set

! Weight of piston/heat-reservoir
! Recommended values:
! wwr=0.005d0   / wqr=1.5d0 for algo=2/3
! wwr=0.005d1   / wqr=1.5d1 for 4
! wwr=0.005d-35             for 5
! wwr=0.005d-33 / wqr=1.5d1 for 6
! wwr=0.005d-28             for 7,8
! wwr=0.005d-33             for 107
      if ((algorithm==2).or.(algorithm==3)) then
         wwr=0.005d0; wqr=1.5d0
      else if (algorithm==4) then
         wwr=0.005d1; wqr=1.5d1
      else if (algorithm==5) then
         wwr=0.005d-35
      else if (algorithm==6) then
         wwr=0.005d-33; wqr=1.5d1
      else if ((algorithm==7).or.(algorithm==8)) then
         wwr=0.005d-28
      else
         wwr=0.005d-33
      endif
      wwr=wwr*stress_weight
      wqr=wqr*temp_weight

      ww=wwr/a**4*wmave
      wq=wqr*wmave*a**2

! Set bookkeeping algorithm
      if (potential_func=='Tersoff') then
         bookkeep_algo = 2
      else if (potential_func=='TersoffNM') then
         bookkeep_algo = 2
      else if (potential_func=='StilWeb') then
         bookkeep_algo = 2
      else if (potential_func=='TersoffBelko') then
         bookkeep_algo = 2
      else if (potential_func=='MEAM') then
         bookkeep_algo = 2
      else
         bookkeep_algo = 1
      endif

! In the case of shell model
      if (potential_func=='Shellmodel') then
         allocate (tmprx(n/2,2),tmpry(n/2,2),tmprz(n/2,2)) 
         allocate (tmpfx(n/2,2),tmpfy(n/2,2),tmpfz(n/2,2)) 
         if ((.not.pbcx).or.(.not.pbcy).or.(.not.pbcz)) then
            print *,'For shell model calculation, only 3D periodic system is supported.'
            stop
         endif
      endif


! writing data
      call writedata

!===  MD routine

      call mkq
      if ((mod(step,nbk)==0).or.(step==step_start)) then
         call out
         if (rpart) then
            call bookkeep_rp
         else
            call bookkeep
         endif
      endif
      step=step+1

! Force and energy calculation
      iii=0
      if (iii==1) then
         print *,'CURVE CHECK'
         open (50,file='potcurve.d')
         do i=0,5
            do j=0,5
               xx(1) = xrmin(1)+(xrmax(1)-xrmin(1))/real(5)*real(i)
               xx(2) = xrmin(2)+(xrmax(2)-xrmin(2))/real(5)*real(j)
               write (50,*) xx(1),xx(2),tfunc(xx,nval)
               print *,i,j,tfunc(xx,nval)
            enddo
            write (50,*)
         enddo
         stop
      endif

      pres=(enkin*2.0d0+virx+viry+virz)/v/3.0d0
      sgm=-pres
      call calcstresstensor
      temp=enkin*2.0d0/3.0d0/dble(n)/bl
      call writedata

! Elastic constants (numerical evaluation): use with care
      if (econstd>0.0) then
         call calcelastic
      endif

!=============================================================================

      write(*,*)
!!! Loop of GA
      do 100 ng=0,ngen
      if(mode(1).eq.0) then
        call getval0
      else
        call getvalb
      endif
      write(*,'(i5$)') ng
      call minfind
      call roulette(val,q,np)
      call select(x0,chrom,newch,q,ns,nbit,np,nval)
      if(mode(2).eq.0) then
         call cross(x0,chrom,newch,ns,nbit,crate,np,nval)
      else
         call mcross(x0,chrom,newch,ns,nbit,crate,np,nval)
      endif
      call mttn(x0,chrom,nbit,mrate,np,nval)
  100 continue
      
      close(1)
      close(2)
      close(3)
      close(4)
      close(7)
      close(8)
      
      write (6,*)
      write (6,'(a16,x,48i1)')   'Best chromosome:'
      write (6,'(a16,x,e20.10)') 'f_min =         ',valmin0
      write (6,'(a16,x,i10)')    'step number =   ',ngbest
      do ival=1,nval
         if (mode(1) == 0) then
            do ibit=1,nbit
               n21(ibit)  = gbest(ibit+nbit*(ival-1))
            enddo
         else
            do ibit=1,nbit
               n21g(ibit) = gbest(ibit+nbit*(ival-1))
            enddo
            call g2b(n21g,n21,nbit)
         endif
      
         call b2d(n21,nbit,n101)
         call getxi(n101,x1,xrmin(ival),xrmax(ival),nbit)
         write (6,'(e20.10,3x,24i1)') x1,(best(ii),ii=1+nbit*(ival-1),nbit*ival)
      enddo
      
      end program pfga
     
      
           
      subroutine init1(x0)
      use fit_variables
!      integer chrom(n,nbit)
      double precision :: x0,r
      
      do 10 i=1,np
      do 10 j=1,nbit*nval
      
      call rand(x0,r)
      
      if(r.ge.0.5d0) then
      chrom(i,j)=1
      else
      chrom(i,j)=0
      endif
      
   10 continue
      
      return
      end
      
      
      
      subroutine getval0
      use fit_variables
      double precision :: x1(nval)
      common ng
      
      do 10 i=1,np

!      write (6,'(i,4x,24i1,x,24i1)') i,(chrom(i,ii),ii=1,nbit*nval)
      do 120 ival=1,nval
      do 12 ibit=1,nbit
         n21(ibit)=chrom(i,ibit+nbit*(ival-1))
 12   continue
!      write (6,'(i,4x,24i1,x,24i1)') ival,(n21(ii),ii=1,nbit)

      call b2d(n21,nbit,n101)
      call getxi(n101,xrmin(ival),xrmax(ival),x1(ival),nbit)
 120  continue
!      print *,chrom(i,1)-chrom(i,25),x1(1),x1(2)
      val(i)=tfunc(x1,nval)
      if(mod(ng,10).eq.0) then
      write(7,'(1h ,i5,3f12.8)') ng,(x1(ii),ii=1,nval)
      endif
      
      if(ng.le.50) then
      write(8,'(1h ,i5,3f12.8)') ng,(x1(ii),ii=1,nval)
      endif
      
   10 continue
      
      if(ng.eq.0) then
      write(1,'('' in subr. getval'')')
      write(1,'(1h ,i3,'':'',24i2,''  f='',f12.8)') &
                   (i,(chrom(i,j),j=1,nbit),val(i),i=1,np)
      write(1,*)
      endif
      
      return
      end
      
      subroutine b2d(n2,nbit,n10)
!
!      transform from binary number to decimal number
!
      dimension n2(nbit)
      n10=0
      do 10 i=1,nbit
      ii=nbit-i
      n10=n10+n2(i)*2**ii
   10 continue
      return
      end
      
      subroutine minfind
        use fit_variables
!      double precision :: val,vx0,valmax,x1,x10,av,vn0,valmin
!      integer chrom(n,nbit),best(nbit)
!      dimension val(n),n21(nbit)
      common ng

      max=1
      min=1
      valmax=-2.0d0
      valmin= 2.0d0
      av=0.0d0
      do 10 i=1,np
      if(val(i).gt.valmax) then
        max=i
        valmax=val(i)
      endif
      if(val(i).lt.valmin) then
        min=i
        valmin=val(i)
      endif
      av=av+val(i)
   10 continue
      
      do 12 ibit=1,nbit
      n21(ibit)=chrom(min,ibit)
   12 continue
      
      call b2d(n21,     nbit,n101)
      
      call getxi(n101,x1,xrmin(min),xrmax(min),nbit)
      
      av=av/dble(np)
      
!      write(2,'(i5,f12.8)') ng,valmin

      write(3,'(i5,f12.8)') ng,av
      
      if(valmax.gt.valmax0) then
         valmax0=valmax
      endif
      if(valmin.lt.valmin0) then
         valmin0=valmin
         ngbest=ng
         do j=1,nbit*nval
            gbest(j)=chrom(min,j)
         enddo
      endif
      
      do j=1,nbit*nval
         best(j)=chrom(min,j)
      enddo
      
      x10=x1
      
      write(4,'(i5,f12.8)') ng,valmin0
      
      write(2,'(i5,f12.8)') ng,val(min)
      if(ng.eq.500) then
      write(1,*) (best(i),i=1,nbit)

      write(1,*) x10,valmin0

      endif
      
      return
      end
      
      
      
      subroutine rand(x,yfl)
      implicit double precision (a-h,o-z)
      x=x*495365.0d0
      if(x) 10,10,20
   10 x=x+2147483648.0d0
   20 ii=idint(x/2147483648.0d0)
      x=x-dfloat(ii)*2147483648.0d0
      yfl=x/2147483648.0d0
      return
      end
      
      
      
      subroutine roulette(val,q,np)
      double precision :: val,q,sum,sigmod
      dimension val(np),q(np)
      common ng
      
      sum=0.0d0
      do 10 i=1,np
      sum=sum+sigmod(val(i))
   10 continue
      
      q(1)=sigmod(val(1))/sum
      do 12 i=2,np
      q(i)=q(i-1)+sigmod(val(i))/sum
   12 continue
      
      if(ng.eq.0) then
      write(1,'('' in subr. roulette'')')
      write(1,'(1h ,i3,2f10.7)') (i,q(i)-q(i-1),q(i),i=1,np)
      write(1,*)
      endif
      
      return
      end
      
      
      subroutine select(x0,chrom,newch,q,ns,nbit,np,nval)
      integer chrom(np,nbit*nval)
      dimension newch(np,nbit*nval),ns(np),q(np)
      double precision :: x0,r,q
      common ng
      
      if(ng.eq.0) then
      write(1,'('' in subr. select'')')
      endif
      
      do 10 i=1,np
      call rand(x0,r)
      do 12 j=1,np
      if(q(j).gt.r) then
      ns(i)=j
      if(ng.eq.0) then
      write(1,'(1h ,2f10.7,i3)') r,q(j),j
      endif
      go to 10
      endif
   12 continue
   10 continue
      
      do 14 i=1,np
      do 14 j=1,nbit*nval
      newch(i,j)=chrom(ns(i),j)
   14 continue
      
      do 16 i=1,np
      do 16 j=1,nbit*nval
      chrom(i,j)=newch(i,j)
   16 continue
      
      if(ng.eq.0) then
      write(1,'(1h ,i3,'':'',24i2,''  ('',i2,'')'')') &
             (i,(chrom(i,j),j=1,nbit),ns(i),i=1,np)
      write(1,*)
      endif
      
      return
      end
      
      
      
      subroutine cross(x0,chrom,newch,ns,nbit,crate,np,nval)
      integer chrom(np,nbit*nval)
      double precision :: x0,r,crate
      dimension newch(np,nbit*nval),ns(np)
      common ng
      
      if(ng.eq.0) then
      write(1,'('' in subr. cross'')')
      endif
      
      nc=0
      do 10 i=1,np
      call rand(x0,r)
      if(ng.eq.0) then
      write(1,'(1h ,f10.7)') r
      endif
      if(r.le.crate) then
      nc=nc+1
      ns(nc)=i
      if(ng.eq.0) then
      write(1,'(1h ,f10.7,i3)') r,i
      endif
      endif
   10 continue
      
      if(mod(nc,2).eq.1) then
      call rand(x0,r)
      nc=nc+1
      ns(nc)=idint(dfloat(np)*r)+1
      endif
      
      do 12 i=1,nc/2
      i1=2*i-1
      i2=2*i
      
      call rand(x0,r)
      icpnt=idint(dfloat(nbit*nval-1)*r)+1
      if(ng.eq.0) then
      write(1,'(1h ,3i5)') ns(i1),ns(i2),icpnt
      endif
      do 14 j=1,icpnt
      newch(i1,j)=chrom(ns(i1),j)
      newch(i2,j)=chrom(ns(i2),j)
   14 continue
      do 16 j=icpnt+1,nbit*nval
      newch(i1,j)=chrom(ns(i2),j)
      newch(i2,j)=chrom(ns(i1),j)
   16 continue
      
      if(ng.eq.0) then
      write(1,'(1h ,i3,'':'',24i2)') ns(i1),(chrom(ns(i1),j),j=1,nbit)
      write(1,'(1h ,i3,'':'',24i2)') ns(i2),(chrom(ns(i2),j),j=1,nbit)
      write(1,'(1h ,i3,'':'',24i2)') ns(i1),(newch(i1,j),j=1,nbit)
      write(1,'(1h ,i3,'':'',24i2)') ns(i2),(newch(i2,j),j=1,nbit)
      endif
      
   12 continue
      
      do 18 i=1,nc
      do 18 j=1,nbit*nval
      chrom(ns(i),j)=newch(i,j)
   18 continue
      
      if(ng.eq.0) then
      write(1,'(1h ,i3,'':'',24i2)') (i,(chrom(i,j),j=1,nbit),i=1,np)
      write(1,*)
      endif
      
      return
      end
      
      
      
      subroutine mttn(x0,chrom,nbit,mrate,np,nval)
      integer chrom(np,nbit*nval)
      double precision :: x0,r,mrate
      common ng
      
      if(ng.eq.0) then
      write(1,'('' in subr. mttn'')')
      endif
      
      do 10 i=1,np
      do 10 j=1,nbit*nval
      
      call rand(x0,r)
      
      if(r.le.mrate) then
      
      if(ng.eq.0) then
      write(1,'(1h ,2i5,f10.7)') i,j,r
      endif
      if(chrom(i,j).eq.1) then
      chrom(i,j)=0
      else
      chrom(i,j)=1
      endif
      
      endif
      
   10 continue
      
      if(ng.eq.0) then
      write(1,'(1h ,i3,'':'',24i2)') (i,(chrom(i,j),j=1,nbit),i=1,np)
      write(1,*)
      endif
      
      return
      end
      
      
      
      
      
      function sigmod(x)
      double precision :: sigmod,x
      
      
      sigmod=1.0d0/(1.0d0+dexp( x))

! For maximum search:
!      sigmod=1.0d0/(1.0d0+dexp(-x))
      
      return
      end
      
      
      
      subroutine g2b(gcode,bcode,nbit)
      integer bcode(nbit),gcode(nbit)
      
      bcode(1)=gcode(1)
      
      do i=2,nbit
      if(bcode(i-1).eq.gcode(i)) then
      bcode(i)=0
      else
      bcode(i)=1
      endif
      enddo
      
      return
      end
      
      
      
      subroutine mcross(x0,chrom,newch,ns,nbit,crate,np,nval)
      integer chrom(np,nbit*nval)
      double precision :: x0,r,crate
      dimension newch(np,nbit*nval),ns(np)
      common ng
      
      if(ng.eq.0) then
      write(1,'('' in subr. mcross'')')
      endif
      
      nc=0
      do 10 i=1,np
      call rand(x0,r)
      if(ng.eq.0) then
      write(1,'(1h ,f10.7)') r
      endif
      if(r.le.crate) then
      nc=nc+1
      ns(nc)=i
      if(ng.eq.0) then
      write(1,'(1h ,f10.7,i3)') r,i
      endif
      endif
   10 continue
      
      if(mod(nc,2).eq.1) then
      call rand(x0,r)
      nc=nc+1
      ns(nc)=idint(dfloat(np)*r)+1
      endif
      
      do 12 i=1,nc/2
      i1=2*i-1
      i2=2*i
      
      if(ng.eq.0) then
      write(1,'(1h ,2i5)') ns(i1),ns(i2)
      endif
      do 14 j=1,nbit*nval
      call rand(x0,r)
      if(r.ge.0.5d0) then
      newch(i1,j)=chrom(ns(i1),j)
      newch(i2,j)=chrom(ns(i2),j)
      else
      newch(i1,j)=chrom(ns(i2),j)
      newch(i2,j)=chrom(ns(i1),j)
      endif
   14 continue
      
      if(ng.eq.0) then
      write(1,'(1h ,i3,'':'',24i2)') ns(i1),(chrom(ns(i1),j),j=1,nbit)
      write(1,'(1h ,i3,'':'',24i2)') ns(i2),(chrom(ns(i2),j),j=1,nbit)
      write(1,'(1h ,i3,'':'',24i2)') ns(i1),(newch(i1,j),j=1,nbit)
      write(1,'(1h ,i3,'':'',24i2)') ns(i2),(newch(i2,j),j=1,nbit)
      endif
      
   12 continue
      
      do 18 i=1,nc
      do 18 j=1,nbit*nval
      chrom(ns(i),j)=newch(i,j)
   18 continue
      
      if(ng.eq.0) then
      write(1,'(1h ,i3,'':'',24i2)') (i,(chrom(i,j),j=1,nbit),i=1,np)
      write(1,*)
      endif
      
      return
      end
      
      
      subroutine getvalb
      use fit_variables
      double precision :: x1(nval)
      common ng
      
      do 10 i=1,np
      
      do 120 ival=1,nval
      do 12 ibit=1,nbit
         n21g(ibit)=chrom(i,ibit+nbit*(ival-1))
   12 continue
      
      call g2b(n21g,n21,nbit)
      call b2d(n21,nbit,n101)
      call getxi(n101,x1(ival),xrmin(ival),xrmax(ival),nbit)
 120  continue
      val(i)=tfunc(x1,nval)
      if(mod(ng,10).eq.0) then
      write(7,'(1h ,i5,3f12.8)') ng,(x1(ii),ii=1,nval)
      endif
      
      if(ng.le.50) then
      write(8,'(1h ,i5,3f12.8)') ng,(x1(ii),ii=1,nval)
      endif
      
   10 continue
      
      if(ng.eq.0) then
      write(1,'('' in subr. getval'')')
      write(1,'(1h ,i3,'':'',24i2,''  f='',f12.8)') &
                   (i,(chrom(i,j),j=1,nbit),val(i),i=1,np)
      write(1,*)
      endif
      
      return
      end
