!
!   Tersoff potential
!   for Silicon
!   calculation of force, total energy, stress
!
!   Jan. 13 (2006)
!   For using a modified bookkeep algorithm ('DOUBLE') for 3-body potential
!   Feb. 24 (2006)
!   Stress calculation is implemented (satom(=atomic stress) is also available)
!   June 6 (2006)
!   Bug in force calculation fixed, stress calculation enabled

      module tersoff_function
      double precision, allocatable :: zmat(:,:),b(:,:)
      double precision :: terrr, terss, teraa, terbb, terlambda, termu, terbeta
      double precision :: tern, terc, terd, terh, terw
      integer :: term
      end module

      subroutine tersoff
      use tersoff_function
      use variables
      use constants
      use bookkeep_param
      use debug_variables
      implicit double precision (a-h,o-z)
      double precision :: ki
      integer :: istat
      if (.not.allocated(zmat)) allocate(zmat(n,n))
      if (.not.allocated(b)) allocate(b(n,n))
      if (bookkeep_algo /= 2) stop 'Error in tersoff.F: Bookkeep algo is not DOUBLE'
      if (n > 3000) stop 'ERROR: Please use TersoffNM instead.'

!--tersoff
      pi=4.0d0*atan(1.0d0)
      eps=1.0d-20
      ev2j=1.6021892d-19        !eV -> J
      j2ev=1.0d0/ev2j           !J -> eV
      mode=1                    !1:38-14(B,C),2:39-8(Multi),1:Date's type
      mparam=0                  !0:Si(B), 1:Si(C) [see PRB 38 p.9902]
                                !4:Date's potential(B),5:Yasukawa,6:Date's Potential(read from file)
      if (TRIM(potential_arg)=='B') mparam=0
      if (TRIM(potential_arg)=='C') mparam=1
      if (TRIM(potential_arg)=='B2') mparam=2
      if (TRIM(potential_arg)=='Sn') mparam=3

!CCC  FOR SILICON
!CCC  PARAMETER FROM Phys.Rev.B 38-14(1988)p.9902
      if (mparam.eq.0) then     ! Si(B) (for surface structure)
         terrr=2.80d-10
         terss=3.20d-10
         teraa=3.2647d3*ev2j
         terbb=9.5373d1*ev2j
         terlambda=3.2394d10
         termu=1.3258d10
         terbeta=3.3675d-1
         tern=2.2956d1
         term=3
         terc=4.8381d0
         terd=2.0417d0
         terh=0.0d0
         terw=1.0d0
      else if (mparam.eq.1) then ! Si(C) (for elastic property)
         terrr=2.70d-10
         terss=3.00d-10
         teraa=1.8308d3*ev2j
         terbb=4.7118d2*ev2j
         terlambda=2.4799d10
         termu=1.7322d10
         terbeta=1.0999d-6
         tern=7.8734d-1
         term=3
         terc=1.0039d5
         terd=1.6218d1
         terh=-5.9826d-1
         terw=1.0d0
      else if (mparam.eq.2) then ! Si(B*) (for phase transformation)
         terrr=2.65d-10
         terss=2.85d-10
         teraa=3.2647d3*ev2j
         terbb=9.5373d1*ev2j
         terlambda=3.2394d10
         termu=1.3258d10
         terbeta=3.3675d-1
         tern=2.2956d1
         term=3
         terc=4.8381d0
         terd=2.0417d0
         terh=0.0d0
         terw=1.0d0
      else if (mparam.eq.3) then ! Sn (Negami)
         terrr=4.15d-10
         terss=4.45d-10
         teraa=1232.0d0*ev2j
         terbb=130.0d0*ev2j
         terlambda=2.277d10
         termu=1.203d10
         tern=0.9907d0
         terbeta=0.1590d0**(1.0/tern)
         term=1
         terc=312.78d0
         terd=14.489d0
         terh=-0.4604d0
         terw=1.0d0
      else 
         stop 'Please specify mparam'
      endif
!     FOR CARBON
!      terrr(2)=1.80d-10
!      terss(2)=2.10d-10
!      teraa(2)=1.3936d3*ev2j
!      terbb(2)=3.467d2*ev2j
!      terlambda(2)=3.4879d10
!      termu(2)=2.2119d10
!      terbeta(2)=1.5724d-7
!      tern(2)=7.2751d-1
!      term(2)=3
!      terc(2)=3.8049d4
!      terd(2)=4.384d0
!      terh(2)=-5.7058d-1
!      terw(2)=1.0d0
!--

      istat=1

      rc=terss
      rc2=terss**2


!   Virial term reset
      virx=0.0d0; viry=0.0d0; virz=0.0d0
      dmat=0.0d0
      satom=0.0d0               ! Atomic stress

!========================================================
!   Loop for b and z
      zmat=0.0d0
      b=0.0d0
      kns=0
      bloop: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         do 1200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
            rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
            if (rij2.le.eps) go to 1201
            rij=sqrt(rij2)
            fij=fcut(rij,terrr,terss)
            if (fij.le.eps) go to 1201
!--------------calc bij---------------------
            knsk=kns0
            do 1300 kcount=1,kn
               knsk=knsk+1
               k=idpt(knsk,1)
               ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
               xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
               call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
               rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
               rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
               if (rik2.le.eps) go to 1301
               rik=sqrt(rik2)
               fik=fcut(rik,terrr,terss)
               if (fik.le.eps) go to 1301
               rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
               if (rjk2.le.eps) go to 1301
               rjk=sqrt(rjk2)
               costh=(rik2+rij2-rjk2)/(2.0d0*rik*rij)
               g=(1.0d0+terc**2/terd**2 &
               -terc**2/(terd**2+(terh-costh)**2))
               if (mode.eq.1) then
                  zmatp=fik*g*exp(termu**term*(rij-rik)**term)
               else
                  zmatp=fik*g*terw
               endif
               zmat(i,j)=zmat(i,j)+zmatp
 1301           continue
 1300        continue
             if (b(i,j)/=0.0) then
                print *,'### Fatal error ###'
                print *,'Sorry, this Tersoff code does not accept too small cells.'
                print *,'Please prepare a larger cell.'
                stop
             endif
             b(i,j)=(1.0d0+(terbeta*zmat(i,j))**tern)**(-0.5d0/tern)
 1201        continue
 1200     continue
      end do bloop

!   Loop
      enpot=0.0d0
      enpoteach=0.0d0
      fx(1:n)=0.0d0
      fy(1:n)=0.0d0
      fz(1:n)=0.0d0
      kns=0
      iloop: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         do 200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
               rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
               if (rij2.le.eps) go to 201
               rij=sqrt(rij2)
               fij=fcut(rij,terrr,terss)
               dfij=fcutd(rij,terrr,terss)
               if (fij.le.eps) go to 201
               ushtij=-fij*b(i,j)*terbb*exp(-termu*rij)
               urepij=fij*teraa*exp(-terlambda*rij)
               uij=urepij+ushtij
               enpoteach(i)=enpoteach(i)+uij*0.5d0
!!     Atomic stress
!               ad1=0.5d0*(-terlambda)*urepij/rij !! bug fixed 150606
               ad1=0.5d0*( (-terlambda)*urepij + dfij/fij*urepij )/rij
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)+ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)+ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)+ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)+ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)+ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)+ad1*adz*adz
!--------------
               ffx=0.0d0;ffy=0.0d0;ffz=0.0d0
               knsk=kns0
               do 300 kcount=1,kn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(i)-rx(k); yy=ry(i)-ry(k); zz=rz(i)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
                  rkx=rx(i)-xx; rky=ry(i)-yy; rkz=rz(i)-zz
                     rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
                     if (rik2.le.eps) go to 301
                     rik=sqrt(rik2)
                     fik=fcut(rik,terrr,terss)
                     if (fik.le.eps) go to 301
                     rjk2=(rkx-rjx)**2+(rky-rjy)**2+(rkz-rjz)**2
                     if (rjk2.le.eps) go to 301
                     rjk=sqrt(rjk2)
                     costh=(rik2+rij2-rjk2)/(2.0d0*rik*rij)
                     g=(1.0d0+terc**2/terd**2 &
                     -terc**2/(terd**2+(terh-costh)**2))
                     dvdz=terbb*fij*exp(-termu*rij)*b(i,j)*terbeta &
                     **tern*zmat(i,j)**tern/2.0d0/(1.0d0+(terbeta*zmat(i,j)) &
                     **tern)/zmat(i,j)

                     if (mode==1) then
                        dzdr1=dble(term)*termu**term &
                        *(rij-rik)**(term-1) &
                        *fik*exp(termu**term*(rij-rik)**term) &
                        *(1.0d0+terc**2/terd**2 &
                        -terc**2/(terd**2+(terh-costh)**2))
                        dzdc=fik*exp(termu**term*(rij-rik)**term) &
                        *(-2.0d0*terc**2*(terh-costh)) &
                        /((terd**2+(terh-costh)**2)**2)
                     else
                        dzdr1=0.0d0
                        dzdc=fik*terw &
                        *(-2.0d0*terc**2*(terh-costh)) &
                        /((terd**2+(terh-costh)**2)**2)
                     endif
!                     dzdr2=fcutd(rik,terrr,terss)-dzdr1 !! bug fixed 150606
                     dzdr2=fcutd(rik,terrr,terss)*exp(termu**term*(rij-rik)**term) &
                        *(1.0d0+terc**2/terd**2 &
                        -terc**2/(terd**2+(terh-costh)**2))-dzdr1
!                     dcdrx=(rik*costh-rij)/(rik*rij2)*(rjx-rix) &
!                     +(rij*costh-rik)/(rij*rik2)*(rkx-rix)
!                     dcdry=(rik*costh-rij)/(rik*rij2)*(rjy-riy) &
!                     +(rij*costh-rik)/(rij*rik2)*(rky-riy)
!                     dcdrz=(rik*costh-rij)/(rik*rij2)*(rjz-riz) &
!                     +(rij*costh-rik)/(rij*rik2)*(rkz-riz)
                     dxij=rix-rjx;dyij=riy-rjy;dzij=riz-rjz
                     dxki=rkx-rix;dyki=rky-riy;dzki=rkz-riz
                     dcdrx=1.0d0/rik*(dxij/rij+dxki*costh/rik) &
                     -1.0d0/rij*(dxki/rik+dxij*costh/rij)
                     dcdry=1.0d0/rik*(dyij/rij+dyki*costh/rik) &
                     -1.0d0/rij*(dyki/rik+dyij*costh/rij)
                     dcdrz=1.0d0/rik*(dzij/rij+dzki*costh/rik) &
                     -1.0d0/rij*(dzki/rik+dzij*costh/rij)
                     ffx=ffx-dvdz*(dzdr1*dxij/rij-dzdr2*dxki/rik+dzdc*dcdrx)/2.0d0
                     ffy=ffy-dvdz*(dzdr1*dyij/rij-dzdr2*dyki/rik+dzdc*dcdry)/2.0d0
                     ffz=ffz-dvdz*(dzdr1*dzij/rij-dzdr2*dzki/rik+dzdc*dcdrz)/2.0d0
!!     Atomic stress
!                     ad1=0.5d0*fij*terbb*exp(-termu*rij)
                     ad1=0.5d0*dvdz*dzdr1/rij
                     ad2=0.5d0*dvdz*dzdr2/rik
!                     adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
                     satom(i,1,1)=satom(i,1,1)+ad1*dxij*dxij+ad2*dxki*dxki
                     satom(i,1,2)=satom(i,1,2)+ad1*dyij*dxij+ad2*dyki*dxki
                     satom(i,2,2)=satom(i,2,2)+ad1*dyij*dyij+ad2*dyki*dyki
                     satom(i,1,3)=satom(i,1,3)+ad1*dzij*dxij+ad2*dzki*dxki
                     satom(i,2,3)=satom(i,2,3)+ad1*dzij*dyij+ad2*dzki*dyki
                     satom(i,3,3)=satom(i,3,3)+ad1*dzij*dzij+ad2*dzki*dzki

                     dxjk=rkx-rjx;dyjk=rky-rjy;dzjk=rkz-rjz
                     ad0=0.5d0*dvdz*dzdc
                     ad1=-1.0d0/rik/rij
                     ad2=1.0d0/rik/rij-costh/rik**2
                     ad3=1.0d0/rik/rij-costh/rij**2
                     satom(i,1,1)=satom(i,1,1)+ad0*(ad1*dxjk*dxjk+ad2*dxki*dxki+ad3*dxij*dxij)
                     satom(i,1,2)=satom(i,1,2)+ad0*(ad1*dyjk*dxjk+ad2*dyki*dxki+ad3*dyij*dxij)
                     satom(i,2,2)=satom(i,2,2)+ad0*(ad1*dyjk*dyjk+ad2*dyki*dyki+ad3*dyij*dyij)
                     satom(i,1,3)=satom(i,1,3)+ad0*(ad1*dzjk*dxjk+ad2*dzki*dxki+ad3*dzij*dxij)
                     satom(i,2,3)=satom(i,2,3)+ad0*(ad1*dzjk*dyjk+ad2*dzki*dyki+ad3*dzij*dyij)
                     satom(i,3,3)=satom(i,3,3)+ad0*(ad1*dzjk*dzjk+ad2*dzki*dzki+ad3*dzij*dzij)
 301                 continue
 300           continue
!--------------
!               ad1=(dfij-terlambda*fij)*teraa*exp(-terlambda*rij)
!               fx(i)=fx(i)+ad1*(rjx-rix)/rij
!               fy(i)=fy(i)+ad1*(rjy-riy)/rij
!               fz(i)=fz(i)+ad1*(rjz-riz)/rij
!               ad1=(fij*termu-dfij)*terbb*(b(i,j)+b(j,i))/2.0d0*exp(-termu*rij)  !dvdr1
!               fx(i)=fx(i)+ad1*(rjx-rix)/rij
!               fy(i)=fy(i)+ad1*(rjy-riy)/rij
!               fz(i)=fz(i)+ad1*(rjz-riz)/rij
               dvdr1=teraa*exp(-terlambda*rij)*(dfij-terlambda &
               *fij)-b(i,j)*terbb*exp(-termu*rij) &
               *(dfij-termu*fij)
               dvdr2=teraa*exp(-terlambda*rij)*(dfij-terlambda &
               *fij)-b(j,i)*terbb*exp(-termu*rij) &
               *(dfij-termu*fij)
               fx(i)=fx(i)-((dvdr1+dvdr2)*dxij/rij)/2.0d0
               fy(i)=fy(i)-((dvdr1+dvdr2)*dyij/rij)/2.0d0
               fz(i)=fz(i)-((dvdr1+dvdr2)*dzij/rij)/2.0d0
               
               fx(i)=fx(i)+ffx
               fy(i)=fy(i)+ffy
               fz(i)=fz(i)+ffz
!!     Atomic stress
!               ad1=0.5d0*fij*terbb*(b(i,j)+b(j,i))/2.0d0 & !! bug fixed 150606
!               *(termu)*exp(-termu*rij)/rij
               ad1=0.5d0*(fij*termu-dfij)*terbb*(b(i,j)+b(j,i))/2.0d0*exp(-termu*rij)/rij
               adx=rjx-rix; ady=rjy-riy; adz=rjz-riz
               satom(i,1,1)=satom(i,1,1)+ad1*adx*adx
               satom(i,1,2)=satom(i,1,2)+ad1*ady*adx
               satom(i,2,2)=satom(i,2,2)+ad1*ady*ady
               satom(i,1,3)=satom(i,1,3)+ad1*adz*adx
               satom(i,2,3)=satom(i,2,3)+ad1*adz*ady
               satom(i,3,3)=satom(i,3,3)+ad1*adz*adz
 201           continue
 200     continue
      end do iloop

!cccc u_ji/r_i
      kns=0
      iloop2: do i=1,n
         rix=rx(i)
         riy=ry(i)
         riz=rz(i)
         kn=knum(i)
         if (kn.eq.0) cycle
         kns0=kns
         do 2200 jcount=1,kn
            kns=kns+1
            j=idpt(kns,1)
            ix=idpt(kns,2); iy=idpt(kns,3); iz=idpt(kns,4)
            xx=rx(i)-rx(j); yy=ry(i)-ry(j); zz=rz(i)-rz(j)
            call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)
            rjx=rx(i)-xx; rjy=ry(i)-yy; rjz=rz(i)-zz
               rij2=(rjx-rix)**2+(rjy-riy)**2+(rjz-riz)**2
               if (rij2.le.eps) go to 2201
               rij=sqrt(rij2)
               fij=fcut(rij,terrr,terss)
               dfij=fcutd(rij,terrr,terss)  !!! added 160606
               if (fij.le.eps) go to 2201
!--------------
               ffx=0.0d0;ffy=0.0d0;ffz=0.0d0
               kjn=knum(j)
               knsk=0; if (j>0) then; do kcount=1,j-1; knsk=knsk+knum(kcount); enddo; endif
               do 2300 kcount=1,kjn
                  knsk=knsk+1
                  k=idpt(knsk,1)
                  ix=idpt(knsk,2); iy=idpt(knsk,3); iz=idpt(knsk,4)
                  xx=rx(j)-rx(k); yy=ry(j)-ry(k); zz=rz(j)-rz(k)
                  call calc_replica(xx,yy,zz,rc,ix,iy,iz,istat)


!!! N.B. "replica" k-coordinates must be calculated from "replica" j-coordinates,
!!!      not "original" j-coordinates.
!!!      i.e. "rkx=rx(j)-xx; rky=ry(j)-yy; rkz=rz(j)-zz" is wrong.
                  rkx=rjx-xx; rky=rjy-yy; rkz=rjz-zz
                     rik2=(rkx-rix)**2+(rky-riy)**2+(rkz-riz)**2
                     if (rik2.le.eps) go to 2301
                     rik=sqrt(rik2)
                     rjk2=(rkx-rjx)**2 &
                         +(rky-rjy)**2+(rkz-rjz)**2
                     if (rjk2.le.eps) go to 2301
                     rjk=sqrt(rjk2)
                     fjk=fcut(rjk,terrr,terss)
                     if (fjk.le.eps) go to 2301
                     rjk=sqrt(rjk2)
!u_ij/r_i
                     costh=(rjk2+rij2-rik2)/(2.0d0*rjk*rij)
                     g=(1.0d0+terc**2/terd**2 &
                     -terc**2/(terd**2+(terh-costh)**2))
                     fij=fcut(rij,terrr,terss)
                     dvdz=terbb*fij*exp(-termu*rij)*b(j,i)*terbeta &
                     **tern*zmat(j,i)**tern/2.0d0/(1.0d0+(terbeta*zmat(j,i)) &
                     **tern)/zmat(j,i)
                     if (mode==1) then
                        dzdr=termu**term*dble(term) &
                        *(rij-rjk)**(term-1) &
                        *fjk*exp(termu**term*(rij-rjk)**term) &
                        *(1.0d0+terc**2/terd**2 &
                        -terc**2/(terd**2+(terh-costh)**2))
                        dzdc=fjk*exp(termu**term*(rij-rjk)**term) &
                        *(-2.0d0*terc**2*(terh-costh)) &
                        /((terd**2+(terh-costh)**2)**2)
                     else
                        dzdr=0.0d0
                        dzdc=fjk*terw &
                        *(-2.0d0*terc**2*(terh-costh)) &
                        /((terd**2+(terh-costh)**2)**2)
                     endif
                     dcdrx=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                     *(rjx-rix)+1.0d0/(rij*rjk)*(rkx-rix)
                     dcdry=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                     *(rjy-riy)+1.0d0/(rij*rjk)*(rky-riy)
                     dcdrz=(rjk2-rik2-rij2)/(2.0d0*rij2*rij*rjk) &
                     *(rjz-riz)+1.0d0/(rij*rjk)*(rkz-riz)
                     dxij=rix-rjx;dyij=riy-rjy;dzij=riz-rjz
!                     dxjk=rjx-rkx;dyjk=rjy-rky;dzjk=rjz-rkz
!                     dcdrx=-1.0d0/rij*(dxjk/rjk+dxij*costh/rij)
!                     dcdry=-1.0d0/rij*(dyjk/rjk+dyij*costh/rij)
!                     dcdrz=-1.0d0/rij*(dzjk/rjk+dzij*costh/rij)
                     ffx=ffx-dvdz*(dzdr*dxij/rij+dzdc*dcdrx)/2.0d0
                     ffy=ffy-dvdz*(dzdr*dyij/rij+dzdc*dcdry)/2.0d0
                     ffz=ffz-dvdz*(dzdr*dzij/rij+dzdc*dcdrz)/2.0d0
!u_jk/r_i
                     dvdz=terbb*fjk*exp(-termu*rjk)*b(j,k)*terbeta &
                     **tern*zmat(j,k)**tern/2.0d0/(1.0d0+(terbeta*zmat(j,k)) &
                     **tern)/zmat(j,k)
                     if (mode==1) then
!                        dzdr=-dble(term)*termu**term &
!                        *(rjk-rij)**(term-1)*fij*g*exp(termu**term &
!                        *(rjk-rij)**term)
                        dzdr=-dble(term)*termu**term &
                        *(rjk-rij)**(term-1)*fij*g*exp(termu**term & !!! amended 160606
                        *(rjk-rij)**term)+dfij*exp(termu**term*(rjk-rij)**term)*g
                        dzdc=fij*exp(termu**term*(rjk-rij)**term) &
                        *(-2.0d0*terc**2.0*(terh-costh)) &
                        /(terd**2+(terh-costh)**2)**2
                     else
                        dzdr=0.0d0
                        dzdc=fij*terw &
                        *(-2.0d0*terc**2.0*(terh-costh)) &
                        /(terd**2+(terh-costh)**2)**2
                     endif
                     ad5=(rjk2-rik2-rij2)/2.0d0/rij2/rij/rjk
                     ad6=1.0d0/rij/rjk
                     dcdrx=(ad5*(rjx-rix)+ad6*(rkx-rix))
                     dcdry=(ad5*(rjy-riy)+ad6*(rky-riy))
                     dcdrz=(ad5*(rjz-riz)+ad6*(rkz-riz))
                     dxij=rix-rjx;dyij=riy-rjy;dzij=riz-rjz
!                     dxjk=rjx-rkx;dyjk=rjy-rky;dzjk=rjz-rkz
!                     dcdrx=-1.0d0/rij*(dxjk/rjk+dxij*costh/rij)
!                     dcdry=-1.0d0/rij*(dyjk/rjk+dyij*costh/rij)
!                     dcdrz=-1.0d0/rij*(dzjk/rjk+dzij*costh/rij)
                     ffx=ffx-dvdz*(dzdr*dxij/rij+dzdc*dcdrx)/2.0d0
                     ffy=ffy-dvdz*(dzdr*dyij/rij+dzdc*dcdry)/2.0d0
                     ffz=ffz-dvdz*(dzdr*dzij/rij+dzdc*dcdrz)/2.0d0
 2301                continue
 2300          continue
               fx(i)=fx(i)+ffx
               fy(i)=fy(i)+ffy
               fz(i)=fz(i)+ffz
!--------------
 2201          continue
 2200    continue
! 2100 continue
      end do iloop2

!---Etot---
      enpot=0.0d0
      do i=1,n
         enpot=enpot+enpoteach(i)
      enddo

!!     Atomic stress
      do i=1,n
         satom(i,2,1)=satom(i,1,2)
         satom(i,3,1)=satom(i,1,3)
         satom(i,3,2)=satom(i,2,3)
      enddo
      dmat=0.0d0
! The sign of dmat is opposite to stress tensor's (dmat is the "virial" tensor)
      do i=1,n
         dmat(1:3,1:3)=dmat(1:3,1:3)-satom(i,1:3,1:3)
      enddo
      virx=dmat(1,1); viry=dmat(2,2); virz=dmat(3,3)

      satom=satom*dble(n)/v

      end

      double precision function fcut(r,rr,ss)
      implicit double precision (a-h,o-z)
      pi=4.0d0*atan(1.0d0)
      if (r.le.rr) then
         fcut=1.0d0
      else if (r.le.ss) then
         fcut=0.5d0+0.5d0*cos(pi*(r-rr)/(ss-rr))
      else
         fcut=0.0d0
      endif
      end
      double precision function fcutd(r,rr,ss)
      implicit double precision (a-h,o-z)
      pi=4.0d0*atan(1.0d0)
      if ((r.lt.ss).and.(r.gt.rr)) then
         fcutd=-pi/2.0d0*sin(pi*(r-rr)/(ss-rr))/(ss-rr)
      else
         fcutd=0.0d0
      endif
      end
